package cz.cvut.fit.xml.index.tpa.examples;

import cz.cvut.fit.xml.index.tpa.AutomatonFactory;
import cz.cvut.fit.xml.index.tpa.DocumentLoader;
import cz.cvut.fit.xml.index.tpa.TreePathsAutomaton;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeSet;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;

/**
 * Example usage of tpalib-1.0.
 *
 * @author eliska.sestakova@fit.cvut.cz
 */
public class TPAExample2 {

    public static void main(String[] args) throws IOException, JDOMException {
        TreePathsAutomaton automaton;

        String path = "XMark-f0.xml";
        //String path = "XMark-f0.005.xml";
        //String path = "XMark-f0.01.xml";
        //String path = "XMark-f0.1.xml";
        //String path = "XMark-f0.5.xml";
        //String path = "XMark-f1.xml";
        //String path = "dblp.xml";

        /**
         * *********************XMARK queries*********************************
         */
        String query1 = "/site/open_auctions";
        String query2 = "/site/people/person/name";
        String query3 = "/site/regions/europe/item/description/parlist/listitem/text/emph";
        String query4 = "//person//watch";
        String query5 = "//regions//mail//date";
        String query6 = "//site//regions//europe//description//listitem//text//emph";
        String query7 = "/site//open_auction";
        String query8 = "//people/person//watch";
        String query9 = "//regions/europe//item//parlist/listitem//text/emph";

        /**
         * *********************DBLP queries*********************************
         */
        /*
        String query1 = "/dblp/mastersthesis";
        String query2 = "/dblp/inproceedings/title";
        String query3 = "/dblp/inproceedings/title/sup/sup";
        String query4 = "//author";
        String query5 = "//inproceedings//booktitle";
        String query6 = "//dblp//article//author";
        String query7 = "//inproceedings/author";
        String query8 = "//dblp/article//title";
        String query9 = "//dblp/inproceedings//title/sup/sup";
         */
        Document document = (new DocumentLoader()).load(path);

        //if the automaton was built before
        if (new File(path + ".ser").isFile()) {
            //use it
            automaton = (new AutomatonFactory()).deserealize(path + ".ser");
        } else {
            //build it and save it
            TimeManager.start();
            automaton = (new AutomatonFactory()).build(document);
            TimeManager.stop();
            System.out.println("Building time: " + (TimeManager.getTime() / 1000) + "s");
            automaton.serialize(path + ".ser");
        }

        automaton.dump();

        //************************** QUERIES ***********************************
        LinkedList<Element> elements = null;
        TreeSet<Integer> dsubset = null;

        List<String> queries = new ArrayList<>();
        queries.add(query1);
        queries.add(query2);
        queries.add(query3);
        queries.add(query4);
        queries.add(query5);
        queries.add(query6);
        queries.add(query7);
        queries.add(query8);
        queries.add(query9);

        double time;
        for (int i = 0; i < 9; i++) {
            String q = queries.get(i);
            time = 0;
            for (int j = 0; j < 100; j++) {
                TimeManager.start();
                dsubset = automaton.getDSubset(q);
                elements = automaton.evaluate(dsubset);
                TimeManager.stop();
                time += TimeManager.getTime();
            }
            System.out.println("Q" + (i + 1) + " getDsubset() + evaluate() time elapsed: " + time / 100 + " ms");

            time = 0;
            for (int j = 0; j < 1; j++) {
                TimeManager.start();
                elements = automaton.evaluate(q);
                TimeManager.stop();
                time += TimeManager.getTime();
            }
            System.out.println("Q" + (i + 1) + " evaluate() time elapsed: " + time / 100 + " ms");

            time = 0;
            for (int j = 0; j < 1; j++) {
                TimeManager.start();
                dsubset = automaton.getDSubset(q);
                TimeManager.stop();
                time += TimeManager.getTime();
            }
            System.out.println("Q" + (i + 1) + " getDsubset() time elapsed: " + time / 100 + " ms");
            System.out.println(elements.size());
        }

    }
}
