package cz.cvut.fit.xml.index.tpa;

import java.io.Serializable;
import java.util.Objects;

/**
 * Automaton transition label (either child or descendant-or-self axis).
 *
 * @author eliska.sestakova@fit.cvut.cz
 */
class EdgeLabel implements Serializable {

    //name of XML element
    private final String label;
    //prefix of edge label corresponding to one of XPath axes
    private final Axis axis;

    /**
     * Constructs new <code>EdgeLabel</code>.
     *
     * @param label name of an XML element
     * @param axis XPath axis
     */
    public EdgeLabel(String label, Axis axis) {
        this.label = label;
        this.axis = axis;
    }

    @Override
    public String toString() {
        return axis + label;
    }

    /**
     *
     */
    public static enum Axis {

        /**
         * XPath child axis (/)
         */
        CHILD,
        /**
         * XPath descendant-or-self axis (//)
         */
        DESCENDANT_OR_SELF;

        @Override
        public String toString() {
            switch (this) {
                case CHILD:
                    return "/";
                case DESCENDANT_OR_SELF:
                    return "//";
                default:
                    throw new IllegalArgumentException();
            }
        }
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.label);
        hash = 79 * hash + Objects.hashCode(this.axis);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final EdgeLabel other = (EdgeLabel) obj;
        if (!Objects.equals(this.label, other.label)) {
            return false;
        }
        if (this.axis != other.axis) {
            return false;
        }
        return true;
    }
}
